package ar.com.drk.drkspiderjava.processor;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ar.com.drk.drkspiderjava.DRKTools;
import ar.com.drk.drkspiderjava.ProgressListener;
import ar.com.drk.drkspiderjava.model.DRKLink;
import ar.com.drk.drkspiderjava.model.MetaTag;
import ar.com.drk.drkspiderjava.model.WebTreeModel;
import ar.com.drk.drkspiderjava.model.WebTreeNode;

public class HTMLParser extends Parser {
	private static final long serialVersionUID = 2602183879460669165L;

	public HTMLParser(WebTreeNode nodo, WebTreeModel model) {
		super(nodo, model);
	}

	/**
	 * Returns text only from HTML string
	 * @param html string with HTML content
	 * @return text only
	 */
	static public String getTextFromHTML(String html) {
		return html.replaceAll("<script[^>]*>[^<]*</script>", "").replaceAll("<style[^>]*>[^<]*</style>", "").replaceAll("<[^>]+>", "").replaceAll("\\s+", " ");
	}
	
	
	/**
	 * Inicia el procesamiento de la URL.
	 * El parser obtiene los links de la URL con el que fue instanciado.
	 */
	public void parse() {
				
		// Busco links
		findURLs();
	}

	/* (non-Javadoc)
	 * @see ar.com.drk.drkspiderjava.processor.Parser#findURLs()
	 */
	@Override
	protected void findURLs() {
		
		String html = nodeContent(null);
		
		// Anchor
		Matcher regex = Pattern.compile("<a[\\s]+[^>]*>(.*?)(?=</a>)</a>", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(html);
		
		while(regex.find()) {
			String anchor = regex.group(1);
			String link = null;
			Map<String, String> attributes = new HashMap<String, String>();
			Matcher attmatcher = Pattern.compile("(\\b\\w+\\b)\\s*=\\s*(?:\"([^\"]*)\"|'([^']*)'|([^\"'<>\\s]+))", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(regex.group());
			
			while(attmatcher.find()) {
				if ("href".equals(attmatcher.group(1)))
					link = attmatcher.group(2);
				else
					attributes.put(attmatcher.group(1), attmatcher.group(2));
			}
			
			if (link != null  && !link.startsWith("#") && !link.startsWith("mailto:") && !link.startsWith("javascript:")) {
				DRKLink newlink;
				try {
					
					if (link.startsWith("http://") || link.startsWith("https://"))
						newlink = new DRKLink("a", anchor, new URL(link), attributes, nodo, regex.start(), regex.end());
					else {
						String path = (nodo.getURL().getPath().length() > 0)?nodo.getURL().getPath().substring(0, nodo.getURL().getPath().lastIndexOf('/')+1):"/";
						if (link.startsWith("/"))
							newlink = new DRKLink("a", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + link), attributes, nodo, regex.start(), regex.end());
						else
							newlink = new DRKLink("a", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + compactPath(path + link)), attributes, nodo, regex.start(), regex.end());
					}
					
					nodo.addLink(newlink);
				}
				catch(MalformedURLException e) {
					e.printStackTrace();
				}
			}
		}

		// Images
		regex = Pattern.compile("<img[\\s]+[^>]+[/]?>", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(html);
		
		while(regex.find()) {
			DRKLink newlink;
			String anchor = "";
			String link = null;
			Map<String, String> attributes = new HashMap<String, String>();
			Matcher attmatcher = Pattern.compile("(\\b\\w+\\b)\\s*=\\s*(?:\"([^\"]*)\"|'([^']*)'|([^\"'<>\\s]+))", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(regex.group());
			
			while(attmatcher.find()) {
				if ("src".equals(attmatcher.group(1)))
					link = attmatcher.group(2);
				else if ("alt".equals(attmatcher.group(1)))
					anchor = attmatcher.group(2);
				else
					attributes.put(attmatcher.group(1), attmatcher.group(2));
			}
			
			if (link != null) {
				try {
					
					if (link.startsWith("http://") || link.startsWith("https://"))
						newlink = new DRKLink("img", anchor, new URL(link), attributes, nodo, regex.start(), regex.end());
					else {
						String path = (nodo.getURL().getPath().length() > 0)?nodo.getURL().getPath().substring(0, nodo.getURL().getPath().lastIndexOf('/')+1):"/";
						if (link.startsWith("/"))
							newlink = new DRKLink("img", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + link), attributes, nodo, regex.start(), regex.end());
						else
							newlink = new DRKLink("img", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + compactPath(path + link)), attributes, nodo, regex.start(), regex.end());
					}
					
					nodo.addLink(newlink);
				}
				catch(MalformedURLException e) {
					e.printStackTrace();
				}
			}
		}
		
		// Link
		regex = Pattern.compile("<link[\\s]+[^>]+[/]?>", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(html);
		
		while(regex.find()) {
			DRKLink newlink;
			String anchor = "";
			String link = null;
			Map<String, String> attributes = new HashMap<String, String>();
			Matcher attmatcher = Pattern.compile("(\\b\\w+\\b)\\s*=\\s*(?:\"([^\"]*)\"|'([^']*)'|([^\"'<>\\s]+))", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(regex.group());
			
			while(attmatcher.find()) {
				if ("href".equals(attmatcher.group(1)))
					link = attmatcher.group(2);
				else
					attributes.put(attmatcher.group(1), attmatcher.group(2));
			}
			
			if (link != null) {
				try {
					
					if (link.startsWith("http://") || link.startsWith("https://"))
						newlink = new DRKLink("link", anchor, new URL(link), attributes, nodo, regex.start(), regex.end());
					else {
						String path = (nodo.getURL().getPath().length() > 0)?nodo.getURL().getPath().substring(0, nodo.getURL().getPath().lastIndexOf('/')+1):"/";
						if (link.startsWith("/"))
							newlink = new DRKLink("link", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + link), attributes, nodo, regex.start(), regex.end());
						else
							newlink = new DRKLink("link", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + compactPath(path + link)), attributes, nodo, regex.start(), regex.end());
					}
					
					nodo.addLink(newlink);
				}
				catch(MalformedURLException e) {
					e.printStackTrace();
				}
			}
		}
		
		// Script o Style
		regex = Pattern.compile("<(script|style)[\\s]+[^>]+[/]?>", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(html);
		
		while(regex.find()) {
			DRKLink newlink;
			String anchor = "";
			String link = null;
			Map<String, String> attributes = new HashMap<String, String>();
			Matcher attmatcher = Pattern.compile("(\\b\\w+\\b)\\s*=\\s*(?:\"([^\"]*)\"|'([^']*)'|([^\"'<>\\s]+))", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(regex.group());
			
			while(attmatcher.find()) {
				if ("src".equals(attmatcher.group(1)))
					link = attmatcher.group(2);
				else
					attributes.put(attmatcher.group(1), attmatcher.group(2));
			}
			
			if (link != null) {
				try {
					
					if (link.startsWith("http://") || link.startsWith("https://"))
						newlink = new DRKLink(regex.group(1), anchor, new URL(link), attributes, nodo, regex.start(), regex.end());
					else {
						String path = (nodo.getURL().getPath().length() > 0)?nodo.getURL().getPath().substring(0, nodo.getURL().getPath().lastIndexOf('/')+1):"/";
						if (link.startsWith("/"))
							newlink = new DRKLink(regex.group(1), anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + link), attributes, nodo, regex.start(), regex.end());
						else
							newlink = new DRKLink(regex.group(1), anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + compactPath(path + link)), attributes, nodo, regex.start(), regex.end());
					}
					
					nodo.addLink(newlink);
				}
				catch(MalformedURLException e) {
					e.printStackTrace();
				}
			}
		}
		
		// Form
		if (model.isFormTraversing()) {
			regex = Pattern.compile("<form[\\s]+[^>]+>", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(html);
			
			while(regex.find()) {
				DRKLink newlink;
				String anchor = "";
				String link = null;
				Map<String, String> attributes = new HashMap<String, String>();
				Matcher attmatcher = Pattern.compile("(\\b\\w+\\b)\\s*=\\s*(?:\"([^\"]*)\"|'([^']*)'|([^\"'<>\\s]+))", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE).matcher(regex.group());
				
				while(attmatcher.find()) {
					if ("action".equals(attmatcher.group(1)))
						link = attmatcher.group(2);
					else
						attributes.put(attmatcher.group(1), attmatcher.group(2));
				}
				
				if (link != null) {
					try {
						
						if (link.startsWith("http://") || link.startsWith("https://"))
							newlink = new DRKLink("form", anchor, new URL(link), attributes, nodo, regex.start(), regex.end());
						else {
							String path = (nodo.getURL().getPath().length() > 0)?nodo.getURL().getPath().substring(0, nodo.getURL().getPath().lastIndexOf('/')+1):"/";
							if (link.startsWith("/"))
								newlink = new DRKLink("form", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + link), attributes, nodo, regex.start(), regex.end());
							else
								newlink = new DRKLink("form", anchor, new URL(nodo.getURL().getProtocol() + "://" + nodo.getURL().getHost() + compactPath(path + link)), attributes, nodo, regex.start(), regex.end());
						}
						
						nodo.addLink(newlink);
					}
					catch(MalformedURLException e) {
						e.printStackTrace();
					}
				}
			}
		} // form traversing
	}
	
	/* (non-Javadoc)
	 * @see ar.com.drk.drkspiderjava.processor.Parser#getSource(ProgressListener progress)
	 */
	@Override
	public String getSource(ProgressListener progress) {
		return nodeContent(progress);
	}

	@Override
	public void finishedGetContent(String content, boolean ok) {
		getHeaderInfo(content);
		
		if (DRKTools.prefs.getBoolean("show_seo_in_tree", true))
			nodo.createSEOReview(content);
	}
	
	@Override
	public String getParserMIMEType() {
		return "text/html.*";
	}
	
	void getHeaderInfo(String html) {
		try {
			// Title
			Pattern p = Pattern.compile("\\Q<title>\\E([^\\<]+)\\Q</title>\\E", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(html);
			if (m.find())
				nodo.setTitle(m.group(1));
			
			// All meta tags
			p = Pattern.compile("\\Q<meta name=\"\\E([^\"]+)\\Q\" content=\"\\E([^\"]+)", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE);//([^\"]+)
			m = p.matcher(html);
			while(m.find())
				nodo.setMeta(m.group(1), new MetaTag(m.group(1), m.group(2), m.start(), m.end()));

			p = Pattern.compile("\\<link([^\\>]+)", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE);
			m = p.matcher(html);
			while(m.find()) {
				Pattern sp =Pattern.compile("rel\\=\"([^\"]+)\"", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE);
				Matcher sm = sp.matcher(m.group(1));
				sm.find();
				String rel = sm.group(1);
				sp =Pattern.compile("href\\=\"([^\"]+)\"", Pattern.MULTILINE | Pattern.CASE_INSENSITIVE);
				sm = sp.matcher(m.group(1));
				sm.find();
				String href = sm.group(1);
				nodo.setRel(rel, href);
			}
		}
		catch (Exception e) {
			// TODO: handle exception
		}
	}
	
	public String getHTML() {
		return nodo.getContent();
	}
}
